{+-----------------------------------------------------------------------------+
 | Created:     16.12.98
 | Last change: 10.01.99
 | Author:      Primoz Gabrijelcic
 | Description: One unit to bind them - common Register for all mwEdit
 |              components
 | Version:     0.16
 | Copyright (c) 1998 Primoz Gabrijelcic
 | No rights reserved.
 |
 | Thanks to: Brad Stowers
 |
 | Version history:
 |   0.1: first version
 |   0.11: 16.12.1998
 |     - Brad Stowers
 |       - Added customised Font editor.
 |   0.12: 22.12.1998
 |     - Primoz Gabrijelcic
 |       - Added registration for DcjPerlSyn and mwGeneralSyn.
 |   0.13: 23.12.1998
 |     - Brad Stowers
 |       - Added keystroke editor.
 |   0.14:
 |     - James Jacobson
 |       - Moved initialization code to mwKeyCmds to prevent "Invalid property
 |         value" error.
 |     - Primoz Gabrijelcic
 |       - Added registration for cbHPSyn.
 |     - Brad Stowers
 |       - Added keystroke editor.
 |   0.15:
 |     - Michael Trier
 |       - Added registration for DcjJavaSyn. Added Java bitmap to DCR file.
 |   0.16:
 |     - Primoz Gabrijelcic
 |       - Added registration for cwCACSyn. Added CA-Clipper bitmap to DCR file.
 |
 +----------------------------------------------------------------------------+}

unit mwEditReg;
{$DEFINE MWE_COMPILER_2} 
interface

procedure Register;

implementation

{begin}                                                                         //bds 12/16/1998
uses    {DsgnIntf,}                                                             //Shane commented it out
   Dialogs, Forms, Graphics,
  mwKeyCmds, Classes,                                                           //bds 12/23/1998
  mwKeyCmdsEditor,                                                              //bds 12/29/1998
  mwCustomEdit, mwPasSyn, DcjCppSyn, DcjPerlSyn, mwGeneralSyn, cbHPSyn,
  DcjJavaSyn,                                                                   //mt 1/2/1999
  cwCACSyn;                                                                     //gp 1/10/1999

type
  TmwEditorFontProperty = class(TFontProperty)
  public
    procedure Edit; override;
  end;

procedure TmwEditorFontProperty.Edit;
const
  { context ids for the Font editor }
  hcDFontEditor = 25000;
var
  FontDialog: TFontDialog;
begin
  FontDialog := TFontDialog.Create(Application);
  try
    FontDialog.Font := TFont(GetOrdValue);
    FontDialog.HelpContext := hcDFontEditor;
    FontDialog.Options := FontDialog.Options + [fdShowHelp, fdForceFontExist,
       fdFixedPitchOnly];
    if FontDialog.Execute then
      SetOrdValue(Longint(FontDialog.Font));
  finally
    FontDialog.Free;
  end;
end;
{end}                                                                           //bds 12/16/1998

{begin}                                                                         //bds 12/23/1998
type
  TmwEditorCommandProperty = class(TIntegerProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TmwEditorCommandProperty }

procedure TmwEditorCommandProperty.Edit;
begin
  ShowMessage('I''m thinking that this will show a dialog that has a list'#13#10+
     'of all editor commands and a description of them to choose from.');
end;

function TmwEditorCommandProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList, paRevertable];
end;

function TmwEditorCommandProperty.GetValue: string;
begin
  Result := EditorCommandToCodeString(TmwEditorCommand(GetOrdValue));
end;

procedure TmwEditorCommandProperty.GetValues(Proc: TGetStrProc);
begin
  GetEditorCommandValues(Proc);
end;

procedure TmwEditorCommandProperty.SetValue(const Value: string);
var
  NewValue: longint;                                                            //bds 12/29/1998
begin
  if IdentToEditorCommand(Value, NewValue) then
    SetOrdValue(NewValue)
  else
    inherited SetValue(Value);
end;

{end}                                                                           //bds 12/23/1998

procedure Register;
begin
  mwCustomEdit.Register;
  RegisterPropertyEditor(TypeInfo(TFont), TmwCustomEdit,                        //bds 12/16/1998
     'Font', TmwEditorFontProperty);
  RegisterPropertyEditor(TypeInfo(TmwEditorCommand), NIL,                       //bds 12/23/1998
     'Command', TmwEditorCommandProperty);
  RegisterPropertyEditor(TypeInfo(TmwKeystrokes), NIL, '',                      //bds 12/29/1998
     TmwKeyStrokesProperty);
  mwPasSyn.Register;
  DcjCppSyn.Register;
  DcjPerlSyn.Register;                                                          //gp 12/30/1998
  mwGeneralSyn.Register;                                                        //gp 12/30/1998
  cbHPSyn.Register;                                                             //gp 12/30/1998
  DcjJavaSyn.Register;                                                          //mt 1/2/1999
  cwCACSyn.Register;                                                            //gp 1/10/1999
end; { Register }

(* This moved to mwKeyCmds.pas because it has to be there under D3.             //bds 12/29/1998
{begin}                                                                         //bds 12/23/1998
initialization
  RegisterIntegerConsts(TypeInfo(TmwEditorCommand), IdentToEditorCommand,
     EditorCommandToIdent);
{end}                                                                           //bds 12/23/1998
*)                                                                              //bds 12/29/1998
end.
